@@ -2741,7 +2741,7 @@ struct genBstrList g;
 #ifdef BSTRLIB_NOVSNP
 /* This is just a hack.  If you are using a system without a vsnprintf, it is 
    not recommended that bformat be used at all. */
-#define exvsnprintf(r,b,n,f,a) {vsprintf (b,f,a); r = -1;}
+#define exvsnprintf(r,b,n,f,a) {portable_vsprintf (b,f,a); r = -1;}
 #define START_VSNBUFF (256)
 #else
 
@@ -2752,7 +2752,7 @@ extern int vsnprintf (char *buf, size_t count, const char *format, va_list arg);
  */
 #endif
 
-#define exvsnprintf(r,b,n,f,a) {r = vsnprintf (b,n,f,a);}
+#define exvsnprintf(r,b,n,f,a) {r = portable_vsnprintf (b,n,f,a);}
 #endif
 #endif
 
