@@ -1,6 +1,6 @@
 /* 
- * Copyright (C) 2003, 2004, 2005 Mondru AB.
  * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2003, 2004, 2005 Mondru AB.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,10 +76,12 @@ struct app_conn_t {
   void *uplink;                  /* Uplink network interface (Internet) */
   void *dnlink;                  /* Downlink network interface (Wireless) */
 
-  char is_adminsession;
+  uint8_t inuse:1;
+  uint8_t is_adminsession:1;
+  uint8_t uamabort:1;
+  uint8_t uamexit:1;
 
   /* Management of connections */
-  int inuse;
   int unit;
   int dnprot;                    /* Downlink protocol */
 
@@ -96,6 +98,10 @@ struct app_conn_t {
   struct session_params s_params;         /* Session parameters */
   struct session_state  s_state;          /* Session state */
 
+#ifdef HAVE_PATRICIA
+  patricia_tree_t *ptree;
+#endif
+
   /* Radius authentication stuff */
   /* Parameters are initialised whenever a reply to an access request
      is received. */
@@ -144,10 +150,6 @@ struct app_conn_t {
   struct in_addr mask;
   struct in_addr dns1;
   struct in_addr dns2;
-
-  /* UAM information */
-  char uamabort; /* should be bit options */
-  char uamexit;
 };
 
 #define VAL_STRING   0
@@ -157,7 +159,14 @@ struct app_conn_t {
 #define VAL_ULONG64  4
 #define VAL_USHORT   5
 
-int set_env(char *name, char type, void *value, int len);
+typedef enum {
+  ACCT_USER,
+#ifdef ENABLE_GARDENACCOUNTING
+  ACCT_GARDEN,
+#endif
+} acct_type;
+
+void set_env(char *name, char type, void *value, int len);
 
 extern struct app_conn_t *firstfreeconn; /* First free in linked list */
 extern struct app_conn_t *lastfreeconn;  /* Last free in linked list */
@@ -196,9 +205,13 @@ struct app_conn_t * chilli_connect_layer3(struct in_addr *src, struct dhcp_conn_
 int chilli_getconn(struct app_conn_t **conn, uint32_t ip, 
 		   uint32_t nasip, uint32_t nasport);
 
+int chilli_appconn_run(int (*cb)(struct app_conn_t *, void *), void *d);
+
 int chilli_req_attrs(struct radius_t *radius, 
 		     struct radius_packet_t *pack,
+		     acct_type type,
 		     uint32_t service_type,
+		     uint8_t status_type,
 		     uint32_t port,
 		     uint8_t *hismac,
 		     struct in_addr *hisip,
@@ -219,14 +232,16 @@ void chilli_print(bstring s, int listfmt,
 		  struct app_conn_t *appconn,
 		  struct dhcp_conn_t *conn);
 
-int chilli_acct_fromsub(struct app_conn_t *appconn, size_t len);
-int chilli_acct_tosub(struct app_conn_t *appconn, size_t len);
+int chilli_acct_fromsub(struct app_conn_t *appconn, 
+			struct pkt_ipphdr_t *ipph);
+int chilli_acct_tosub(struct app_conn_t *appconn, 
+		      struct pkt_ipphdr_t *ipph);
 
 int terminate_appconn(struct app_conn_t *appconn, int terminate_cause);
 
 void config_radius_session(struct session_params *params, 
 			   struct radius_packet_t *pack, 
-			   struct dhcp_conn_t *dhcpconn,
+			   struct app_conn_t *appconn,
 			   int reconfig);
 
 void session_param_defaults(struct session_params *params);
@@ -249,6 +264,8 @@ uint32_t mainclock_diffu(time_t past);
 
 pid_t chilli_fork(uint8_t type, char *name);
 
+void child_killall(int sig);
+
 #define CHILLI_PROC        0
 #define CHILLI_PROC_DAEMON 1
 #define CHILLI_PROC_REDIR  2
@@ -256,6 +273,8 @@ pid_t chilli_fork(uint8_t type, char *name);
 
 #ifdef ENABLE_PROXYVSA
 int radius_addvsa(struct radius_packet_t *pack, struct redir_state *state);
+int chilli_learn_location(uint8_t *loc, int loclen, 
+			  struct app_conn_t *appconn, char force);
 #endif
 
 #ifdef HAVE_NETFILTER_COOVA
@@ -285,12 +304,15 @@ void GenerateAuthenticatorResponse(u_char *Password, int PasswordLen,
 int chilli_getconn_byroute(struct app_conn_t **conn, int idx);
 #endif
 
+uint8_t* chilli_called_station(struct session_state *state);
+
 int chilli_cmd(struct cmdsock_request *req, bstring s, int sock);
 
 int chilli_handle_signal(void *ctx, int fd);
 void chilli_freeconn();
 
-int runscript(struct app_conn_t *appconn, char* script);
+int runscript(struct app_conn_t *appconn, char* script,
+	      char *loc, char *oloc);
 
 /* utils.c */
 int statedir_file(char *dst, int dlen, char *file, char *deffile);
