@@ -1,6 +1,5 @@
 /* 
  * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
- * Copyright (C) 2006 PicoPoint B.V.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,6 +62,11 @@ static cmd_info commands[] = {
 #endif
 #ifdef ENABLE_STATFILE
   { CMDSOCK_STATUSFILE,    "statusfile",    NULL },
+#endif
+  { CMDSOCK_ADD_GARDEN,  "addgarden", NULL },
+  { CMDSOCK_REM_GARDEN,  "remgarden", NULL },
+#ifdef ENABLE_INSPECT
+  { CMDSOCK_INSPECT,  "inspect", NULL },
 #endif
   { 0, NULL, NULL }
 };
@@ -90,100 +94,106 @@ static struct cmdsock_request request;
 static struct cmd_arguments args[] = {
   { "ip", 
     CMDSOCK_FIELD_IPV4, 
-    sizeof(request.sess.ip),
-    &request.sess.ip,
+    sizeof(request.ip),
+    &request.ip,
     "IP address of session to perform action on", 0, 0 },
   { "mac",
     CMDSOCK_FIELD_MAC, 
-    sizeof(request.sess.mac),
-    request.sess.mac,
+    sizeof(request.mac),
+    request.mac,
     "MAC address of session to perform action on", 0, 0 },
   { "sessionid",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.sessionid),
-    request.sess.sessionid,
+    sizeof(request.d.sess.sessionid),
+    request.d.sess.sessionid,
     "Session-id of session to perform action on", 0, 0 },
   { "username",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.username),
-    request.sess.username,
+    sizeof(request.d.sess.username),
+    request.d.sess.username,
     "Username to use in RADIUS 'login' or authorization", 0, 0 },
   { "password",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.password),
-    request.sess.password,
+    sizeof(request.d.sess.password),
+    request.d.sess.password,
     "Password to be used for 'login' command", 0, 0 },
   { "sessiontimeout",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.sessiontimeout),
-    &request.sess.params.sessiontimeout,
+    sizeof(request.d.sess.params.sessiontimeout),
+    &request.d.sess.params.sessiontimeout,
     "Max session time (in seconds)", 0, 0 },
   { "idletimeout",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.idletimeout),
-    &request.sess.params.idletimeout,
+    sizeof(request.d.sess.params.idletimeout),
+    &request.d.sess.params.idletimeout,
     "Max idle time (in seconds)", 0, 0 },
   { "interiminterval",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.interim_interval),
-    &request.sess.params.interim_interval,
+    sizeof(request.d.sess.params.interim_interval),
+    &request.d.sess.params.interim_interval,
     "Accounting interim interval",  0, 0 },
   { "maxoctets",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.maxtotaloctets),
-    &request.sess.params.maxtotaloctets,
+    sizeof(request.d.sess.params.maxtotaloctets),
+    &request.d.sess.params.maxtotaloctets,
     "Max input + output octets (bytes)", 0, 0 },
   { "maxinputoctets",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.maxinputoctets),
-    &request.sess.params.maxinputoctets,
+    sizeof(request.d.sess.params.maxinputoctets),
+    &request.d.sess.params.maxinputoctets,
     "Max input octets (bytes)", 0, 0 },
   { "maxoutputoctets",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.maxoutputoctets),
-    &request.sess.params.maxoutputoctets,
+    sizeof(request.d.sess.params.maxoutputoctets),
+    &request.d.sess.params.maxoutputoctets,
     "Max output octets (bytes)", 0, 0 },
   { "maxbwup", 
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.bandwidthmaxup),
-    &request.sess.params.bandwidthmaxup,
+    sizeof(request.d.sess.params.bandwidthmaxup),
+    &request.d.sess.params.bandwidthmaxup,
     "Max bandwidth up", 0, 0 },
   { "maxbwdown",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.bandwidthmaxdown),
-    &request.sess.params.bandwidthmaxdown,
+    sizeof(request.d.sess.params.bandwidthmaxdown),
+    &request.d.sess.params.bandwidthmaxdown,
     "Max bandwidth down", 0, 0 },
   { "splash",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.params.url),
-    &request.sess.params.url,
+    sizeof(request.d.sess.params.url),
+    &request.d.sess.params.url,
     "Set splash page",
-    &request.sess.params.flags, REQUIRE_UAM_SPLASH },
+    &request.d.sess.params.flags, REQUIRE_UAM_SPLASH },
 #ifdef ENABLE_REDIRINJECT
   { "inject",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.params.url),
-    &request.sess.params.url,
+    sizeof(request.d.sess.params.url),
+    &request.d.sess.params.url,
     "Inject url flag",
-    &request.sess.params.flags, UAM_INJECT_URL | REQUIRE_UAM_AUTH },
+    &request.d.sess.params.flags, UAM_INJECT_URL | REQUIRE_UAM_AUTH },
 #endif
   { "url",
     CMDSOCK_FIELD_STRING, 
-    sizeof(request.sess.params.url),
-    &request.sess.params.url,
+    sizeof(request.d.sess.params.url),
+    &request.d.sess.params.url,
     "Set redirect url",
-    &request.sess.params.flags, REQUIRE_REDIRECT },
+    &request.d.sess.params.flags, REQUIRE_REDIRECT },
 #ifdef ENABLE_MULTIROUTE
   { "routeidx",
     CMDSOCK_FIELD_INTEGER, 
-    sizeof(request.sess.params.routeidx),
-    &request.sess.params.routeidx,
+    sizeof(request.d.sess.params.routeidx),
+    &request.d.sess.params.routeidx,
     "Route interface index",  0, 0 },
 #endif
   { "noacct",
     CMDSOCK_FIELD_NONE, 0, 0,
     "No accounting flag",
-    &request.sess.params.flags, NO_ACCOUNTING },
+    &request.d.sess.params.flags, NO_ACCOUNTING },
+  { "data",
+    CMDSOCK_FIELD_STRING, 
+    sizeof(request.d.data),
+    &request.d.data,
+    "Text configuration line",
+    0, 1 },
   /* more... */
 };
 
@@ -227,8 +237,7 @@ static int usage(char *program) {
   fprintf(stderr, "  socket = full path to UNIX domain socket (e.g. /var/run/chilli.sock)\n");
 
   fprintf(stderr, "  Available Commands:\n    ");
-  for (i=0; i < count; i++) {
-    if (!commands[i].command) break;
+  for (i=0; commands[i].command; i++) {
     fprintf(stderr, "%s%s", 
 	    i > 0 ? ", " : "",
 	    commands[i].command);
@@ -256,6 +265,100 @@ static void timeout_alarm(int signum) {
   exit(1);
 }
 
+static int process_args(int argc, char *argv[], int argidx) {
+  int c = argc - argidx;
+  char is_data = 0;
+  
+  while(c > 0) {
+    int i;
+    
+    for (i=0; i < count; i++) {
+      
+      if (!strcmp(argv[argidx], args[i].name)) {
+	
+	if (args[i].flag) {
+	  *(args[i].flag) |= args[i].flagbit;
+	} else if (args[i].flagbit == 1) {
+	  if (is_data == 0) 
+	    is_data = 1;
+	  else {
+	    fprintf(stderr, "data can only be once and by itself\n");
+	    return argidx;
+	  }
+	} else {
+	  if (i > 1) {
+	    if (is_data == 1) {
+	      fprintf(stderr, "data can only be once and by itself\n");
+	      return argidx;
+	    }
+	    is_data = -1;
+	  }
+	}
+	
+	if (c == 1 && args[i].length) {
+	  fprintf(stderr, "Argument %s requires a value\n", argv[argidx]);
+	  return usage(argv[0]);
+	}
+	
+	switch(args[i].type) {
+	case CMDSOCK_FIELD_NONE:
+	  break;
+	case CMDSOCK_FIELD_MAC:
+	  parse_mac(((uint8_t *)args[i].field), argv[argidx+1]);
+	  break;
+	case CMDSOCK_FIELD_STRING:
+	  safe_strncpy(((char *)args[i].field), argv[argidx+1], args[i].length);
+	  break;
+	case CMDSOCK_FIELD_INTEGER:
+	  switch(args[i].length) {
+	  case 1:
+	    *((uint8_t *)args[i].field) |= (uint8_t)atoi(argv[argidx+1]);
+	    break;
+	  case 2:
+	    *((uint16_t *)args[i].field) |= (uint16_t)atoi(argv[argidx+1]);
+	    break;
+	  case 4:
+	    *((uint32_t *)args[i].field) |= (uint32_t)atol(argv[argidx+1]);
+	    break;
+	  case 8:
+	    *((uint64_t *)args[i].field) |= (uint64_t)atol(argv[argidx+1]);
+	    break;
+	  }
+	  break;
+	case CMDSOCK_FIELD_IPV4: 
+	  {
+	    struct in_addr ip;
+	    if (!inet_aton(argv[argidx+1], &ip)) {
+	      fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx+1]);
+	      return usage(argv[0]);
+	    }
+	    ((struct in_addr *)args[i].field)->s_addr = ip.s_addr;
+	    break;
+	  }
+	}
+	break;
+      }
+    }
+    
+    if (i == count) {
+      if (request.type == CMDSOCK_LOGOUT)
+	break;
+      fprintf(stderr, "Unknown argument: %s\n", argv[argidx]);
+      return usage(argv[0]);
+    }
+    
+    if (args[i].length) {
+      c -= 2;
+      argidx += 2;
+    } else {
+      c --;
+      argidx ++;
+    }
+  }
+
+  return argidx;
+}
+
 int main(int argc, char **argv) {
   /*
    *   chilli_query [ -s <unix-socket> ] <command> [<argument>]
@@ -289,9 +392,9 @@ int main(int argc, char **argv) {
   set_signal(SIGALRM, timeout_alarm);
   
   memset(&itval, 0, sizeof(itval));
-  itval.it_interval.tv_sec = QUERY_TIMEOUT;
+  itval.it_interval.tv_sec = query_timeout;
   itval.it_interval.tv_usec = 0; 
-  itval.it_value.tv_sec = QUERY_TIMEOUT;
+  itval.it_value.tv_sec = query_timeout;
   itval.it_value.tv_usec = 0; 
   
   if (setitimer(ITIMER_REAL, &itval, NULL)) {
@@ -333,86 +436,17 @@ int main(int argc, char **argv) {
       request.type = commands[s].type;
       
       switch(request.type) {
-	
+
+#ifdef ENABLE_INSPECT
+      case CMDSOCK_INSPECT:
+#endif
       case CMDSOCK_LOGIN:
       case CMDSOCK_LOGOUT:
       case CMDSOCK_UPDATE:
       case CMDSOCK_AUTHORIZE:
-	{
-	  int c = argc - argidx;
-	  
-	  while(c > 0) {
-	    int i;
-	    
-	    for (i=0; i<count; i++) {
-	      
-	      if (!strcmp(argv[argidx],args[i].name)) {
-		
-		if (args[i].flag) {
-		  *(args[i].flag) |= args[i].flagbit;
-		}
-		
-		if (c == 1 && args[i].length) {
-		  fprintf(stderr, "Argument %s requires a value\n", argv[argidx]);
-		  return usage(argv[0]);
-		}
-		
-		switch(args[i].type) {
-		case CMDSOCK_FIELD_NONE:
-		  break;
-		case CMDSOCK_FIELD_MAC:
-		  parse_mac(((uint8_t *)args[i].field), argv[argidx+1]);
-		  break;
-		case CMDSOCK_FIELD_STRING:
-		  safe_strncpy(((char *)args[i].field), argv[argidx+1], args[i].length);
-		  break;
-		case CMDSOCK_FIELD_INTEGER:
-		  switch(args[i].length) {
-		  case 1:
-		    *((uint8_t *)args[i].field) |= (uint8_t)atoi(argv[argidx+1]);
-		    break;
-		  case 2:
-		    *((uint16_t *)args[i].field) |= (uint16_t)atoi(argv[argidx+1]);
-		    break;
-		  case 4:
-		    *((uint32_t *)args[i].field) |= (uint32_t)atol(argv[argidx+1]);
-		    break;
-		  case 8:
-		    *((uint64_t *)args[i].field) |= (uint64_t)atol(argv[argidx+1]);
-		    break;
-		  }
-		  break;
-		case CMDSOCK_FIELD_IPV4: 
-		  {
-		    struct in_addr ip;
-		    if (!inet_aton(argv[argidx+1], &ip)) {
-		      fprintf(stderr, "Invalid IP Address: %s\n", argv[argidx+1]);
-		      return usage(argv[0]);
-		    }
-		    ((struct in_addr *)args[i].field)->s_addr = ip.s_addr;
-		    break;
-		  }
-		}
-		break;
-	      }
-	    }
-
-	    if (i == count) {
-	      if (request.type == CMDSOCK_LOGOUT)
-		break;
-	      fprintf(stderr, "Unknown argument: %s\n", argv[argidx]);
-	      return usage(argv[0]);
-	    }
-
-	    if (args[i].length) {
-	      c -= 2;
-	      argidx += 2;
-	    } else {
-	      c --;
-	      argidx ++;
-	    }
-	  }
-	}
+      case CMDSOCK_ADD_GARDEN:
+      case CMDSOCK_REM_GARDEN:
+	argidx = process_args(argc, argv, argidx);
 	if (request.type != CMDSOCK_LOGOUT || argidx >= argc)
 	  break;
 	/* else, drop through */
@@ -425,7 +459,7 @@ int main(int argc, char **argv) {
 	    return usage(argv[0]);
 	  }
 
-	  if (parse_mac(request.sess.mac, argv[argidx]))
+	  if (parse_mac(request.mac, argv[argidx]))
 	    return usage(argv[0]);
 	  
 	  /* do another switch to pick up param configs for authorize */
@@ -446,7 +480,7 @@ int main(int argc, char **argv) {
 	    return usage(argv[0]);
 	  }
 	 
-	  request.sess.ip.s_addr = ip.s_addr;
+	  request.ip.s_addr = ip.s_addr;
 	}
 	break;
 #ifdef ENABLE_MULTIROUTE
@@ -482,10 +516,10 @@ int main(int argc, char **argv) {
 	  }
 
 	  for (i = 0; i < PKT_ETH_ALEN; i++) 
-	    request.sess.mac[i] = temp[i];
+	    request.mac[i] = temp[i];
 
 	  argidx++;
-	  request.sess.params.routeidx = atoi(argv[argidx]);
+	  request.d.sess.params.routeidx = atoi(argv[argidx]);
 
 	  if (request.type != CMDSOCK_ROUTE_GW)
 	    request.type = CMDSOCK_ROUTE_SET;
@@ -510,7 +544,6 @@ int main(int argc, char **argv) {
     struct sockaddr_in s;
     int blen = sizeof(struct pkt_chillihdr_t);
     uint8_t b[blen];
-    int len;
 
     int fd = socket(AF_INET, SOCK_DGRAM, 0);
 
@@ -526,9 +559,9 @@ int main(int argc, char **argv) {
     s.sin_port = htons(10203);
     s.sin_addr.s_addr = htonl(INADDR_BROADCAST);
     
-    len = safe_sendto(fd, b, blen, 0, 
-		      (struct sockaddr *)&s, 
-		      sizeof(struct sockaddr_in));
+    (void) safe_sendto(fd, b, blen, 0, 
+		       (struct sockaddr *)&s, 
+		       sizeof(struct sockaddr_in));
 
   } else {
 #endif
