@@ -39,64 +39,6 @@ static void setup_filter(net_interface *iface);
 static int default_sndbuf = 0;
 static int default_rcvbuf = 0;
 
-int safe_accept(int fd, struct sockaddr *sa, socklen_t *lenptr) {
-  int ret;
-  do {
-    ret = accept(fd, sa, lenptr);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_connect(int s, struct sockaddr *sock, size_t len) {
-  int ret;
-  do {
-    ret = connect(s, sock, len);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_read(int s, void *b, size_t blen) {
-  int ret;
-  do {
-    ret = read(s, b, blen);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_recvfrom(int sockfd, void *buf, size_t len, int flags,
-		  struct sockaddr *src_addr, socklen_t *addrlen) {
-  int ret;
-  do {
-    ret = recvfrom(sockfd, buf, len, flags, src_addr, addrlen);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_write(int s, void *b, size_t blen) {
-  int ret;
-  do {
-    ret = write(s, b, blen);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_sendto(int s, const void *b, size_t blen, int flags,
-		const struct sockaddr *dest_addr, socklen_t addrlen) {
-  int ret;
-  do {
-    ret = sendto(s, b, blen, flags, dest_addr, addrlen);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
-int safe_close (int fd) {
-  int ret;
-  do {
-    ret = close(fd);
-  } while (ret == -1 && errno == EINTR);
-  return ret;
-}
-
 int dev_set_flags(char const *dev, int flags) {
   struct ifreq ifr;
   int fd;
@@ -219,7 +161,8 @@ int dev_set_address(char const *devname, struct in_addr *address,
   return dev_set_flags(devname, IFF_UP | IFF_RUNNING); 
 }
 
-int net_init(net_interface *netif, char *ifname, uint16_t protocol, int promisc, uint8_t *mac) {
+int net_init(net_interface *netif, char *ifname, 
+	     uint16_t protocol, int promisc, uint8_t *mac) {
 
   if (ifname) {
     memset(netif, 0, sizeof(net_interface));
@@ -312,8 +255,10 @@ int net_close(net_interface *netif) {
 int net_select_init(select_ctx *sctx) {
 #if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
   sctx->efd = epoll_create(MAX_SELECT);
-  if (sctx->efd <=0)
+  if (sctx->efd <= 0) {
+    log_err(errno,"!! could not create epoll !!");
     return -1;
+  }
 #endif  
   return 0;
 }
@@ -337,7 +282,7 @@ int net_select_prepare(select_ctx *sctx) {
   i = 0; /* for compiler */
 #else
 #ifdef USING_POLL
-  for (i=0; i < MAX_SELECT; i++) {
+  for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd) {
       sctx->pfds[i].fd = sctx->desc[i].fd;
       sctx->pfds[i].events = 0;
@@ -351,15 +296,16 @@ int net_select_prepare(select_ctx *sctx) {
   fd_zero(&sctx->rfds);
   fd_zero(&sctx->wfds);
   fd_zero(&sctx->efds);
-  for (i=0; i < MAX_SELECT; i++) {
-    if (!sctx->desc[i].fd && sctx->desc[i].evts & SELECT_RESET) {
-      sctx->desc[i].cb(&sctx->desc[i], -1);
-    }
+  for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd) {
-      if (sctx->desc[i].evts & SELECT_READ)
+      if (sctx->desc[i].evts & SELECT_READ) {
 	fd_set(sctx->desc[i].fd, &sctx->rfds);
+	fd_set(sctx->desc[i].fd, &sctx->efds);
+      }
       if (sctx->desc[i].evts & SELECT_WRITE)
 	fd_set(sctx->desc[i].fd, &sctx->wfds);
+    } else if (sctx->desc[i].evts & SELECT_RESET) {
+      sctx->desc[i].cb(&sctx->desc[i], -1);
     }
   }
 #endif
@@ -371,6 +317,8 @@ int net_select_dereg(select_ctx *sctx, int oldfd) {
   int i;
   for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd == oldfd) {
+      for (; i < sctx->count - 1; i++)
+	memcpy(&sctx->desc[i], &sctx->desc[i+1], sizeof(select_fd));
       memset(&sctx->desc[i], 0, sizeof(select_fd));
       sctx->count--;
       return 0;
@@ -447,7 +395,7 @@ int net_select_zero(select_ctx *sctx) {
 #ifdef USING_POLL
   int i;
   memset(&sctx->pfds, 0, sizeof(sctx->pfds));
-  for (i=0; i < MAX_SELECT; i++) {
+  for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd) {
       sctx->pfds[i].fd = sctx->desc[i].fd;
       sctx->pfds[i].events = 0;
@@ -469,11 +417,15 @@ int net_select_zero(select_ctx *sctx) {
 
 int net_select_rmfd(select_ctx *sctx, int fd) {
 #if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
+  struct epoll_event event;
+  memset(&event, 0, sizeof(event));
+  event.data.fd = fd;
   log_dbg("epoll rm %d", fd);
   /*
    */
-  if (epoll_ctl(sctx->efd, EPOLL_CTL_DEL, fd, 0))
-    log_err(errno, "Failed to remove fd %d", fd);
+  if (epoll_ctl(sctx->efd, EPOLL_CTL_DEL, fd, &event))
+    log_err(errno, "Failed to remove fd %d (%d)", 
+	    fd, sctx->efd);
 #endif
   return 0;
 }
@@ -485,11 +437,12 @@ int net_select_addfd(select_ctx *sctx, int fd, int evts) {
   event.data.fd = fd;
   if (evts & SELECT_READ) event.events |= EPOLLIN;
   if (evts & SELECT_WRITE) event.events |= EPOLLOUT;
-  log_dbg("epoll add %d", fd);
+  log_dbg("epoll add %d (%d)", fd, sctx->efd);
   /*
    */
   if (epoll_ctl(sctx->efd, EPOLL_CTL_ADD, fd, &event))
-    log_err(errno, "Failed to watch fd");
+    log_err(errno, "Failed to add fd %d (%d)",
+	    fd, sctx->efd);
 #endif
   return 0;
 }
@@ -539,27 +492,43 @@ int net_select_fd(select_ctx *sctx, int fd, char evts) {
 
 int net_select(select_ctx *sctx) {
   int status;
-#ifdef USING_POLL
-#ifdef HAVE_SYS_EPOLL_H
-  status = epoll_wait(sctx->efd, sctx->events, MAX_SELECT, 1000);
-#else
-  status = poll(sctx->pfds, sctx->count, 1000);
-#endif
-#else
-  sctx->idleTime.tv_sec = 1;
-  sctx->idleTime.tv_usec = 0;
+
   do {
 
+#ifdef USING_POLL
+#ifdef HAVE_SYS_EPOLL_H
+    status = epoll_wait(sctx->efd, sctx->events, MAX_SELECT, 1000);
+#else
+    status = poll(sctx->pfds, sctx->count, 1000);
+#endif
+#else
+    sctx->idleTime.tv_sec = 1;
+    sctx->idleTime.tv_usec = 0;
+    
     status = select(sctx->maxfd + 1, 
 		    &sctx->rfds, 
 		    &sctx->wfds, 
 		    &sctx->efds, 
 		    &sctx->idleTime);
-
-    if (status == -1) net_select_prepare(sctx); /* reset */
-
-  } while (status == -1 && errno == EINTR);
+    
+#if(0)
+    if (status) log_dbg("select() == %d", status);
+    {int i;
+      for (i=0; i<FD_SETSIZE; i++)
+	if (FD_ISSET(i, &sctx->rfds))
+	  log_dbg("rfds[%d]",i);
+      for (i=0; i<FD_SETSIZE; i++)
+	if (FD_ISSET(i, &sctx->wfds))
+	  log_dbg("wfds[%d]",i);
+      for (i=0; i<FD_SETSIZE; i++)
+	if (FD_ISSET(i, &sctx->efds))
+	  log_dbg("efds[%d]",i);
+    }
 #endif
+    
+    if (status == -1) net_select_prepare(sctx); /* reset */
+#endif
+  } while (status == -1 && errno == EINTR);
   return status;
 }
 
@@ -567,46 +536,52 @@ int net_select_read_fd(select_ctx *sctx, int fd) {
 #ifdef USING_POLL
   int idx;
 #ifdef HAVE_SYS_EPOLL_H
-  for (idx=0; idx < MAX_SELECT; idx++)
+  for (idx=0; idx < sctx->count; idx++)
     if (sctx->events[idx].data.fd == fd) {
-      return (sctx->events[idx].events & EPOLLIN) != 0;
+      if (sctx->events[idx].events & EPOLLIN)
+	return 1;
     }
-  return 0;
 #else
-  for (idx=0; idx < MAX_SELECT; idx++)
+  for (idx=0; idx < sctx->count; idx++)
     if (sctx->pfds[idx].fd == fd)
-      return (sctx->pfds[idx].events & POLLIN) != 0;
-  return 0;
+      if (sctx->pfds[idx].events & POLLIN)
+	return 1;
 #endif
 #else
   if (FD_ISSET(fd, &sctx->efds))
     return -1;
-  return FD_ISSET(fd, &sctx->rfds);
+  if (FD_ISSET(fd, &sctx->rfds))
+    return 1;
 #endif
+  return 0;
 }
 
 int net_select_write_fd(select_ctx *sctx, int fd) {
 #ifdef USING_POLL
   int idx;
 #ifdef HAVE_SYS_EPOLL_H
-  for (idx=0; idx < MAX_SELECT; idx++)
+  for (idx=0; idx < sctx->count; idx++)
     if (sctx->events[idx].data.fd == fd) {
 #if(_debug_ > 1)
       log_dbg("write %d", (sctx->events[idx].events & EPOLLOUT) != 0);
 #endif
-      return (sctx->events[idx].events & EPOLLOUT) != 0;
+      if (sctx->events[idx].events & EPOLLOUT)
+	return 1;
     }
-  return 0;
 #else
-  for (idx=0; idx < MAX_SELECT; idx++)
+  for (idx=0; idx < sctx->count; idx++)
     if (sctx->pfds[idx].fd == fd) {
-      return (sctx->pfds[idx].events & POLLOUT) != 0;
+      if (sctx->pfds[idx].events & POLLOUT)
+	return 1;
     }
-  return 0;
 #endif
 #else
-  return FD_ISSET(fd, &sctx->wfds);
+  if (FD_ISSET(fd, &sctx->efds))
+    return -1;
+  if (FD_ISSET(fd, &sctx->wfds))
+    return 1;
 #endif
+  return 0;
 }
 
 int net_run_selected(select_ctx *sctx, int status) {
@@ -617,7 +592,7 @@ int net_run_selected(select_ctx *sctx, int status) {
     sfd->cb(sfd->ctx, sfd->idx);
   }
 #else
-  for (i=0; i < MAX_SELECT; i++) {
+  for (i=0; i < sctx->count; i++) {
     if (sctx->desc[i].fd) {
 #ifdef USING_POLL
       char has_read = !!(sctx->pfds[i].revents & POLLIN);
@@ -651,9 +626,11 @@ struct netpcap {
 };
 
 static void 
-net_pcap_read(u_char *user, const struct pcap_pkthdr *hdr, const u_char *bytes) {
+net_pcap_read(u_char *user, const struct pcap_pkthdr *hdr, 
+	      const u_char *bytes) {
   struct netpcap *np = (struct netpcap *)user;
-  if (!bytes || hdr->caplen < sizeof(struct pkt_ethhdr_t) || hdr->caplen > np->dlen)
+  if (!bytes || hdr->caplen < sizeof(struct pkt_ethhdr_t) ||
+      hdr->caplen > np->dlen)
     np->read = -1;
   else {
     memcpy(np->d, bytes, hdr->caplen);
@@ -662,7 +639,8 @@ net_pcap_read(u_char *user, const struct pcap_pkthdr *hdr, const u_char *bytes)
 }
 
 static void 
-net_pcap_handler(u_char *user, const struct pcap_pkthdr *hdr, const u_char *bytes) {
+net_pcap_handler(u_char *user, const struct pcap_pkthdr *hdr,
+		 const u_char *bytes) {
   struct netpcap *np = (struct netpcap *)user;
   if (!bytes || hdr->caplen < sizeof(struct pkt_ethhdr_t))
     np->read = -1;
@@ -683,7 +661,7 @@ net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx) {
     np.d = ctx;
     np.dlen = 0;
     np.read = 0;
-    
+
     cnt = pcap_dispatch(netif->pd, 1, net_pcap_handler, (u_char *)&np);
     
     return cnt ? np.read : -1;
@@ -697,19 +675,31 @@ net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx) {
 #endif
 
   {
+    struct pkt_buffer pb;
     uint8_t packet[PKT_MAX_LEN];
-    ssize_t length = net_read_eth(netif, packet, sizeof(packet));
+    ssize_t length;
+    pkt_buffer_init(&pb, packet, sizeof(packet), PKT_BUFFER_IPOFF);
+    length = net_read_eth(netif, 
+			  pkt_buffer_head(&pb), 
+			  pkt_buffer_size(&pb));
     if (length <= 0) return length;
-    return func(ctx, packet, length);
+    pb.length = length;
+    return func(ctx, &pb);
   }
 }
 
 ssize_t 
 net_read_dispatch(net_interface *netif, net_handler func, void *ctx) {
+  struct pkt_buffer pb;
   uint8_t packet[PKT_MAX_LEN];
-  ssize_t length = safe_read(netif->fd, packet, sizeof(packet));
+  ssize_t length;
+  pkt_buffer_init(&pb, packet, sizeof(packet), PKT_BUFFER_IPOFF);
+  length = safe_read(netif->fd, 
+		     pkt_buffer_head(&pb), 
+		     pkt_buffer_size(&pb));
   if (length <= 0) return length;
-  return func(ctx, packet, length);
+  pb.length = length;
+  return func(ctx, &pb);
 }
 
 ssize_t 
@@ -739,23 +729,73 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 #endif
 
   if (netif->fd) {
-#if defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
-    len = safe_read(netif->fd, d, dlen);
+
+#if defined(__linux__)
+    struct sockaddr_ll s_addr;
+
+#if defined(HAVE_LINUX_TPACKET_AUXDATA_TP_VLAN_TCI)
+    struct iovec iov;
+    struct msghdr msg;
+    struct cmsghdr *cmsg;
+    union {
+      struct cmsghdr cmsg;
+      char buf[CMSG_SPACE(sizeof(struct tpacket_auxdata))];
+    } cmsg_buf;
+#ifdef ENABLE_IEEE8021Q
+    struct vlan_tag {
+      u_int16_t tpid; 
+      u_int16_t tci; 
+    };
+#endif
+    msg.msg_name = &s_addr;
+    msg.msg_namelen = sizeof(s_addr);
+    msg.msg_iov = &iov;
+    msg.msg_iovlen = 1;
+    msg.msg_control = &cmsg_buf;
+    msg.msg_controllen = sizeof(cmsg_buf);
+    msg.msg_flags = 0;
+    
+    iov.iov_len = dlen;
+    iov.iov_base = d;
 #else
     int addr_len;
-    struct sockaddr_ll s_addr;
-    memset (&s_addr, 0, sizeof (struct sockaddr_ll));
-    addr_len = sizeof (s_addr);
+#endif
 
+    memset (&s_addr, 0, sizeof (struct sockaddr_ll));
+
+#if defined(HAVE_LINUX_TPACKET_AUXDATA_TP_VLAN_TCI) 
+    len = safe_recvmsg(netif->fd, &msg, MSG_TRUNC);
+#else
+    addr_len = sizeof (s_addr);
     len = safe_recvfrom(netif->fd, d, dlen,
 			MSG_DONTWAIT | MSG_TRUNC, 
 			(struct sockaddr *) &s_addr, 
 			(socklen_t *) &addr_len);
-    if (len > dlen) {
-      log_warn(0, "data truncated, sending ICMP error %d/%d", 
-	       len, dlen);
-      len = dlen;
+#endif
+    if (len < 0) {
+
+      log_err(errno, "could not read packet");
+
+    } else {
+
+      if (len == 0)
+	log_dbg("read zero, enable ieee8021q?");
+      
+      if (len > dlen) {
+	log_warn(0, "data truncated, sending ICMP error %d/%d", 
+		 len, dlen);
+	len = dlen;
+      }
     }
+
+#elif defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
+
+    len = safe_read(netif->fd, d, dlen);
+
+#else
+
+    len = safe_read(netif->fd, d, dlen);
+
 #endif
 
     if (len < 0) {
@@ -763,6 +803,47 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 	      netif->fd, dlen, netif->mtu, len);
       return -1;
     }
+
+#if defined(HAVE_LINUX_TPACKET_AUXDATA_TP_VLAN_TCI) && defined(ENABLE_IEEE8021Q)
+    if (_options.ieee8021q) {
+      for (cmsg = CMSG_FIRSTHDR(&msg); cmsg; cmsg = CMSG_NXTHDR(&msg, cmsg)) {
+	struct tpacket_auxdata *aux;
+	struct vlan_tag *tag;
+	unsigned int ulen;
+	
+	if (cmsg->cmsg_len < CMSG_LEN(sizeof(struct tpacket_auxdata)) ||
+	    cmsg->cmsg_level != SOL_PACKET ||
+	    cmsg->cmsg_type != PACKET_AUXDATA)
+	  continue;
+	
+	aux = (struct tpacket_auxdata *)CMSG_DATA(cmsg);
+	if (aux->tp_vlan_tci == 0)
+	  continue;
+	
+	ulen = len > iov.iov_len ? iov.iov_len : len;
+	
+	if (ulen < 2 * PKT_ETH_ALEN ||
+	    len >= PKT_MAX_LEN - 4) {
+	  log_err(0, "bad pkt length to add 802.1q header %d/%d",
+		  ulen, len);
+	  break;
+	}
+	
+#if(_debug_ > 1)
+	log_dbg("adding 8021q header from auxdata");
+#endif
+	
+	memmove(d + (2 * PKT_ETH_ALEN) + 4, 
+		d + (2 * PKT_ETH_ALEN), 
+		len - (2 * PKT_ETH_ALEN));
+	
+	tag = (struct vlan_tag *)(d + 2 * PKT_ETH_ALEN);
+	tag->tpid = htons(ETH_P_8021Q);
+	tag->tci = htons(aux->tp_vlan_tci);
+	len += 4;
+      }
+    }
+#endif
   }
   
   return len;
@@ -771,15 +852,23 @@ net_read_eth(net_interface *netif, void *d, size_t dlen) {
 ssize_t net_write(int sock, void *d, size_t dlen) {
   int left = dlen;
   int w, off = 0;
+
   while (left > 0) {
-    w = safe_write(sock, d + off, left);
+    w = safe_send(sock, d + off, left, 
+#ifdef MSG_NOSIGNAL
+		  MSG_NOSIGNAL
+#else
+		  0
+#endif
+		  );
     if (w < 0) {
-      log_err(errno, "write()");
-      return off ? off : -1;
+      log_err(errno, "safe_send(%d, d+%d,%d)", sock, off, left);
+      return (errno == EWOULDBLOCK || errno == EAGAIN) ? off : -1;
     }
     left -= w;
     off += w;
   }
+
   return off;
 }
 
@@ -1159,9 +1248,7 @@ int net_open_eth(net_interface *netif) {
   }
 
   if (ifr.ifr_hwaddr.sa_family == ARPHRD_ETHER) {
-
     netif->flags |= NET_ETHHDR;
-
     if ((netif->flags & NET_USEMAC) == 0) {
       memcpy(netif->hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
     } else if (_options.dhcpmacset) {
@@ -1175,7 +1262,8 @@ int net_open_eth(net_interface *netif) {
   }
   
   if (netif->hwaddr[0] & 0x01) {
-    log_err(0, "Ethernet has broadcast or multicast address: %.16s", netif->devname);
+    log_err(0, "Ethernet has broadcast or multicast address: %.16s",
+	    netif->devname);
   }
   
   /* Get the current interface address, network, and any destination address */
@@ -1188,7 +1276,8 @@ int net_open_eth(net_interface *netif) {
     return -1;
   }
   if (ifr.ifr_mtu > PKT_BUFFER) {
-    log_err(0, "MTU is larger than PKT_BUFFER: %d > %d", ifr.ifr_mtu, PKT_BUFFER);
+    log_err(0, "MTU is larger than PKT_BUFFER: %d > %d", 
+	    ifr.ifr_mtu, PKT_BUFFER);
     return -1;
   }
   netif->mtu = ifr.ifr_mtu;
@@ -1202,8 +1291,45 @@ int net_open_eth(net_interface *netif) {
   
   log_dbg("device %s ifindex %d", netif->devname, netif->ifindex);
 
-#ifndef USING_MMAP
+#ifdef ENABLE_IPV6
+  {
+    struct ifaddrs *ifaddr, *ifa;
+    char host[NI_MAXHOST];
+    int family, s;
+    if (getifaddrs(&ifaddr) == 0) {
+      for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+	if (!ifa->ifa_addr) continue;
+	family = ifa->ifa_addr->sa_family;
 
+	log_dbg("%s  address family: %d%s",
+		ifa->ifa_name, family,
+		(family == AF_PACKET) ? " (AF_PACKET)" :
+		(family == AF_INET) ?   " (AF_INET)" :
+		(family == AF_INET6) ?  " (AF_INET6)" : "");
+
+	if (/*family == AF_INET || */family == AF_INET6 &&
+	    !strcmp(netif->devname, ifa->ifa_name)) {
+	  struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)ifa->ifa_addr;
+	  memcpy(&netif->address_v6, &in6->sin6_addr, 
+		 sizeof(struct in6_addr));
+	  s = getnameinfo(ifa->ifa_addr,
+			  /*(family == AF_INET) ? 
+			    sizeof(struct sockaddr_in) :*/
+			  sizeof(struct sockaddr_in6),
+			  host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+	  if (s != 0) {
+	    log_dbg("getnameinfo() failed: %s\n", strerror(s));
+	  } else {
+	    log_dbg("address: <%s>\n", host);
+	  }
+	}
+      }
+      freeifaddrs(ifaddr);
+    }
+  }
+#endif
+
+#ifndef USING_MMAP
   /* Set interface in promisc mode */
   if (netif->flags & NET_PROMISC) {
 
@@ -1253,6 +1379,14 @@ int net_open_eth(net_interface *netif) {
   netif->dest.sll_ifindex = netif->ifindex;
 #endif
 
+#if defined(HAVE_LINUX_TPACKET_AUXDATA_TP_VLAN_TCI) 
+  option = 1;
+  if (setsockopt(netif->fd, SOL_PACKET, PACKET_AUXDATA, &option,
+		 sizeof(option)) == -1 && errno != ENOPROTOOPT) {
+    log_err(errno, "auxdata");
+  }
+#endif
+
 #ifdef USING_MMAP
   setup_rings2(netif);
   /*setup_filter(netif);*/
@@ -1345,7 +1479,6 @@ int net_open_eth(net_interface *netif) {
   /* Find suitable device */
   for (devnum = 0; devnum < 255; devnum++) { /* TODO 255 */ 
     safe_snprintf(devname, sizeof(devname), "/dev/bpf%d", devnum);
-    devname[sizeof(devname)] = 0;
     if ((netif->fd = open(devname, O_RDWR)) >= 0) break;
     if (errno != EBUSY) break;
   } 
@@ -1382,7 +1515,8 @@ int net_open_eth(net_interface *netif) {
   }
   
   if (netif->hwaddr[0] & 0x01) {
-    log_err(0, "Ethernet has broadcast or multicast address: %.16s", netif->devname);
+    log_err(0, "Ethernet has broadcast or multicast address: %.16s", 
+	    netif->devname);
     return -1;
   }
 
@@ -1449,6 +1583,7 @@ static int rx_ring(net_interface *iface, net_handler func, void *ctx) {
   was_drop = 0;
   for (cnt = 0; cnt < iface->rx_ring.cnt; ++cnt) {
     data = h = iface->rx_ring.frames[iface->rx_ring.idx];
+
     if (!h->tp_status)
       break;
     
