@@ -1,6 +1,5 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2006 PicoPoint B.V.
  * Copyright (C) 2007-2011 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -50,7 +49,7 @@ int option_aton(struct in_addr *addr, struct in_addr *mask,
   c = sscanf(pool, "%u.%u.%u.%u/%u.%u.%u.%u",
 	     &a1, &a2, &a3, &a4,
 	     &m1, &m2, &m3, &m4);
-
+  
   switch (c) {
   case 4:
     mask->s_addr = htonl(0xffffff00);
@@ -60,7 +59,7 @@ int option_aton(struct in_addr *addr, struct in_addr *mask,
       log_err(0, "Invalid mask");
       return -1; /* Invalid mask */
     }
-    mask->s_addr = htonl(0xffffffff << (32 - m1));
+    mask->s_addr = m1 > 0 ? htonl(0xffffffff << (32 - m1)) : 0;
     break;
   case 8:
     if (m1 >= 256 ||  m2 >= 256 || m3 >= 256 || m4 >= 256) {
@@ -115,7 +114,7 @@ static int opt_run(int argc, char **argv, int reload) {
 
   log_dbg("(Re)processing options [%s]", file);
 
-  if ((status = fork()) < 0) {
+  if ((status = safe_fork()) < 0) {
     log_err(errno, "fork() returned -1!");
     return -1;
   }
@@ -277,8 +276,15 @@ int options_fromfd(int fd, bstring bt) {
 #ifdef ENABLE_IEEE8021Q
   if (!option_s_l(bt, &o.vlanupdate)) return 0;
 #endif
+#ifdef ENABLE_PROXYVSA
+  if (!option_s_l(bt, &o.locationupdate)) return 0;
+#endif
 
   if (!option_s_l(bt, &o.radiussecret)) return 0;
+#ifdef ENABLE_LARGELIMITS
+  if (!option_s_l(bt, &o.radiusacctsecret)) return 0;
+  if (!option_s_l(bt, &o.radiusadmsecret)) return 0;
+#endif
   if (!option_s_l(bt, &o.radiusnasid)) return 0;
   if (!option_s_l(bt, &o.radiuslocationid)) return 0;
   if (!option_s_l(bt, &o.radiuslocationname)) return 0;
@@ -286,6 +292,12 @@ int options_fromfd(int fd, bstring bt) {
   if (!option_s_l(bt, &o.proxysecret)) return 0;
   
   if (!option_s_l(bt, &o.dhcpif)) return 0;
+#ifdef ENABLE_MULTILAN
+  for (i=0; i < MAX_MOREIF; i++) {
+    if (!option_s_l(bt, &o.moreif[i].dhcpif)) return 0;
+    if (!option_s_l(bt, &o.moreif[i].vlan)) return 0;
+  }
+#endif
   if (!option_s_l(bt, &o.routeif)) return 0;
   if (!option_s_l(bt, &o.peerkey)) return 0;
 
@@ -326,9 +338,6 @@ int options_fromfd(int fd, bstring bt) {
 #ifdef ENABLE_MODULES
   if (!option_s_l(bt, &o.moddir)) return 0;
 #endif
-#ifdef ENABLE_REDIRINJECT
-  if (!option_s_l(bt, &o.inject)) return 0;
-#endif
 
   if (!option_s_l(bt, &o.adminuser)) return 0;
   if (!option_s_l(bt, &o.adminpasswd)) return 0;
@@ -343,12 +352,21 @@ int options_fromfd(int fd, bstring bt) {
 
   if (!option_s_l(bt, &o.uamaliasname)) return 0;
   if (!option_s_l(bt, &o.uamhostname)) return 0;
-  
+
+#ifdef ENABLE_REDIRINJECT
+  if (!option_s_l(bt, &o.inject)) return 0;
+  if (!option_s_l(bt, &o.inject_ext)) return 0;
+#endif
+
   for (i=0; i < MAX_UAM_DOMAINS; i++) {
     if (!option_s_l(bt, &o.uamdomains[i])) 
       return 0;
   }
 
+#ifdef EX_OPTIONS_LOAD
+#include EX_OPTIONS_LOAD
+#endif
+
 #ifdef ENABLE_CHILLIREDIR
   for (i = 0; i < MAX_REGEX_PASS_THROUGHS; i++) {
 #if defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
@@ -452,8 +470,16 @@ int options_save(char *file, bstring bt) {
 #ifdef ENABLE_IEEE8021Q
   if (!option_s_s(bt, &o.vlanupdate)) return 0;
 #endif
+#ifdef ENABLE_PROXYVSA
+  if (!option_s_s(bt, &o.locationupdate)) return 0;
+#endif
 
   if (!option_s_s(bt, &o.radiussecret)) return 0;
+#ifdef ENABLE_LARGELIMITS
+  if (!option_s_s(bt, &o.radiusacctsecret)) return 0;
+  if (!option_s_s(bt, &o.radiusadmsecret)) return 0;
+#endif
+
   if (!option_s_s(bt, &o.radiusnasid)) return 0;
   if (!option_s_s(bt, &o.radiuslocationid)) return 0;
   if (!option_s_s(bt, &o.radiuslocationname)) return 0;
@@ -461,6 +487,12 @@ int options_save(char *file, bstring bt) {
   if (!option_s_s(bt, &o.proxysecret)) return 0;
 
   if (!option_s_s(bt, &o.dhcpif)) return 0;
+#ifdef ENABLE_MULTILAN
+  for (i=0; i < MAX_MOREIF; i++) {
+    if (!option_s_s(bt, &o.moreif[i].dhcpif)) return 0;
+    if (!option_s_s(bt, &o.moreif[i].vlan)) return 0;
+  }
+#endif
   if (!option_s_s(bt, &o.routeif)) return 0;
   if (!option_s_s(bt, &o.peerkey)) return 0;
 
@@ -501,9 +533,6 @@ int options_save(char *file, bstring bt) {
 #ifdef ENABLE_MODULES
   if (!option_s_s(bt, &o.moddir)) return 0;
 #endif
-#ifdef ENABLE_REDIRINJECT
-  if (!option_s_s(bt, &o.inject)) return 0;
-#endif
 
   if (!option_s_s(bt, &o.adminuser)) return 0;
   if (!option_s_s(bt, &o.adminpasswd)) return 0;
@@ -519,11 +548,20 @@ int options_save(char *file, bstring bt) {
   if (!option_s_s(bt, &o.uamaliasname)) return 0;
   if (!option_s_s(bt, &o.uamhostname)) return 0;
 
+#ifdef ENABLE_REDIRINJECT
+  if (!option_s_s(bt, &o.inject)) return 0;
+  if (!option_s_s(bt, &o.inject_ext)) return 0;
+#endif
+
   for (i = 0; i < MAX_UAM_DOMAINS; i++) {
     if (!option_s_s(bt, &o.uamdomains[i])) 
       return 0;
   }
 
+#ifdef EX_OPTIONS_SAVE
+#include EX_OPTIONS_SAVE
+#endif
+
   oldmask = umask(022);
 
   fd = open(file, O_RDWR | O_TRUNC | O_CREAT, 0666);
